@@ -380,7 +380,7 @@ class OpenSSHTransport(Transport):
         elif self.host.ssh_password:
             password_file = os.path.join(self.control_dir.path, 'password')
             with open(password_file, 'w') as f:
-                os.fchmod(f.fileno(), 600)
+                os.fchmod(f.fileno(), 0o600)
                 f.write(self.host.ssh_password)
                 f.write('\n')
             argv = ['sshpass', '-f', password_file] + argv
